@@ -283,13 +283,33 @@ namespace dxvk {
     std::string description = options->customDeviceDesc.empty()
       ? std::string(deviceProp.deviceName)
       : options->customDeviceDesc;
-    
-    // XXX nvapi workaround for a lot of Unreal Engine 4 games
-    if (options->customVendorId < 0 && options->customDeviceId < 0
-     && options->nvapiHack && deviceProp.vendorID == uint16_t(DxvkGpuVendor::Nvidia)) {
-      Logger::info("DXGI: NvAPI workaround enabled, reporting AMD GPU");
-      deviceProp.vendorID = uint16_t(DxvkGpuVendor::Amd);
-      deviceProp.deviceID = 0x67df; /* RX 480 */
+
+    if (options->customVendorId < 0) {
+      uint16_t fallbackVendor = 0xdead;
+      uint16_t fallbackDevice = 0xbeef;
+
+      if (!options->hideAmdGpu) {
+        // AMD RX 6700XT
+        fallbackVendor = uint16_t(DxvkGpuVendor::Amd);
+        fallbackDevice = 0x73df;
+      } else if (!options->hideNvidiaGpu) {
+        // Nvidia RTX 3060
+        fallbackVendor = uint16_t(DxvkGpuVendor::Nvidia);
+        fallbackDevice = 0x2487;
+      }
+
+      bool hideGpu = (deviceProp.vendorID == uint16_t(DxvkGpuVendor::Nvidia) && options->hideNvidiaGpu)
+                  || (deviceProp.vendorID == uint16_t(DxvkGpuVendor::Amd) && options->hideAmdGpu)
+                  || (deviceProp.vendorID == uint16_t(DxvkGpuVendor::Intel) && options->hideIntelGpu);
+
+      if (hideGpu) {
+        deviceProp.vendorID = fallbackVendor;
+
+        if (options->customDeviceId < 0)
+          deviceProp.deviceID = fallbackDevice;
+
+        Logger::info(str::format("DXGI: Hiding actual GPU, reporting vendor ID 0x", std::hex, deviceProp.vendorID, ", device ID ", deviceProp.deviceID));
+      }
     }
     
     // Convert device name
