@@ -183,12 +183,17 @@ namespace dxvk {
     HWND                            m_window;
     DXGI_SWAP_CHAIN_DESC1           m_desc;
     DXGI_SWAP_CHAIN_FULLSCREEN_DESC m_descFs;
-    UINT                            m_presentCount;
+    UINT                            m_presentId;
 
     Com<IDXGIVkSwapChain>           m_presenter;
+    Com<IDXGIVkSwapChain1>          m_presenter1;
     
     HMONITOR                        m_monitor;
+    bool                            m_monitorHasOutput = true;
+    bool                            m_frameStatisticsDisjoint = true;
     wsi::DxvkWindowState            m_windowState;
+
+    uint32_t                        m_globalHDRStateSerial = 0;
     
     HRESULT EnterFullscreenMode(
             IDXGIOutput1            *pTarget);
@@ -216,6 +221,8 @@ namespace dxvk {
     
     void ReleaseMonitorData();
 
+    void UpdateGlobalHDRState();
+
   };
   
 }
