@@ -165,187 +165,108 @@ namespace dxvk {
     return queues;
   }
 
+#define CHECK_FEATURE_NEED(feature) \
+  (m_deviceFeatures.feature         \
+       || !required.feature)
 
   bool DxvkAdapter::checkFeatureSupport(const DxvkDeviceFeatures& required) const {
-    return (m_deviceFeatures.core.features.robustBufferAccess
-                || !required.core.features.robustBufferAccess)
-        && (m_deviceFeatures.core.features.fullDrawIndexUint32
-                || !required.core.features.fullDrawIndexUint32)
-        && (m_deviceFeatures.core.features.imageCubeArray
-                || !required.core.features.imageCubeArray)
-        && (m_deviceFeatures.core.features.independentBlend
-                || !required.core.features.independentBlend)
-        && (m_deviceFeatures.core.features.geometryShader
-                || !required.core.features.geometryShader)
-        && (m_deviceFeatures.core.features.tessellationShader
-                || !required.core.features.tessellationShader)
-        && (m_deviceFeatures.core.features.sampleRateShading
-                || !required.core.features.sampleRateShading)
-        && (m_deviceFeatures.core.features.dualSrcBlend
-                || !required.core.features.dualSrcBlend)
-        && (m_deviceFeatures.core.features.logicOp
-                || !required.core.features.logicOp)
-        && (m_deviceFeatures.core.features.multiDrawIndirect
-                || !required.core.features.multiDrawIndirect)
-        && (m_deviceFeatures.core.features.drawIndirectFirstInstance
-                || !required.core.features.drawIndirectFirstInstance)
-        && (m_deviceFeatures.core.features.depthClamp
-                || !required.core.features.depthClamp)
-        && (m_deviceFeatures.core.features.depthBiasClamp
-                || !required.core.features.depthBiasClamp)
-        && (m_deviceFeatures.core.features.fillModeNonSolid
-                || !required.core.features.fillModeNonSolid)
-        && (m_deviceFeatures.core.features.depthBounds
-                || !required.core.features.depthBounds)
-        && (m_deviceFeatures.core.features.wideLines
-                || !required.core.features.wideLines)
-        && (m_deviceFeatures.core.features.largePoints
-                || !required.core.features.largePoints)
-        && (m_deviceFeatures.core.features.alphaToOne
-                || !required.core.features.alphaToOne)
-        && (m_deviceFeatures.core.features.multiViewport
-                || !required.core.features.multiViewport)
-        && (m_deviceFeatures.core.features.samplerAnisotropy
-                || !required.core.features.samplerAnisotropy)
-        && (m_deviceFeatures.core.features.textureCompressionETC2
-                || !required.core.features.textureCompressionETC2)
-        && (m_deviceFeatures.core.features.textureCompressionASTC_LDR
-                || !required.core.features.textureCompressionASTC_LDR)
-        && (m_deviceFeatures.core.features.textureCompressionBC
-                || !required.core.features.textureCompressionBC)
-        && (m_deviceFeatures.core.features.occlusionQueryPrecise
-                || !required.core.features.occlusionQueryPrecise)
-        && (m_deviceFeatures.core.features.pipelineStatisticsQuery
-                || !required.core.features.pipelineStatisticsQuery)
-        && (m_deviceFeatures.core.features.vertexPipelineStoresAndAtomics
-                || !required.core.features.vertexPipelineStoresAndAtomics)
-        && (m_deviceFeatures.core.features.fragmentStoresAndAtomics
-                || !required.core.features.fragmentStoresAndAtomics)
-        && (m_deviceFeatures.core.features.shaderTessellationAndGeometryPointSize
-                || !required.core.features.shaderTessellationAndGeometryPointSize)
-        && (m_deviceFeatures.core.features.shaderImageGatherExtended
-                || !required.core.features.shaderImageGatherExtended)
-        && (m_deviceFeatures.core.features.shaderStorageImageExtendedFormats
-                || !required.core.features.shaderStorageImageExtendedFormats)
-        && (m_deviceFeatures.core.features.shaderStorageImageMultisample
-                || !required.core.features.shaderStorageImageMultisample)
-        && (m_deviceFeatures.core.features.shaderStorageImageReadWithoutFormat
-                || !required.core.features.shaderStorageImageReadWithoutFormat)
-        && (m_deviceFeatures.core.features.shaderStorageImageWriteWithoutFormat
-                || !required.core.features.shaderStorageImageWriteWithoutFormat)
-        && (m_deviceFeatures.core.features.shaderUniformBufferArrayDynamicIndexing
-                || !required.core.features.shaderUniformBufferArrayDynamicIndexing)
-        && (m_deviceFeatures.core.features.shaderSampledImageArrayDynamicIndexing
-                || !required.core.features.shaderSampledImageArrayDynamicIndexing)
-        && (m_deviceFeatures.core.features.shaderStorageBufferArrayDynamicIndexing
-                || !required.core.features.shaderStorageBufferArrayDynamicIndexing)
-        && (m_deviceFeatures.core.features.shaderStorageImageArrayDynamicIndexing
-                || !required.core.features.shaderStorageImageArrayDynamicIndexing)
-        && (m_deviceFeatures.core.features.shaderClipDistance
-                || !required.core.features.shaderClipDistance)
-        && (m_deviceFeatures.core.features.shaderCullDistance
-                || !required.core.features.shaderCullDistance)
-        && (m_deviceFeatures.core.features.shaderFloat64
-                || !required.core.features.shaderFloat64)
-        && (m_deviceFeatures.core.features.shaderInt64
-                || !required.core.features.shaderInt64)
-        && (m_deviceFeatures.core.features.shaderInt16
-                || !required.core.features.shaderInt16)
-        && (m_deviceFeatures.core.features.shaderResourceResidency
-                || !required.core.features.shaderResourceResidency)
-        && (m_deviceFeatures.core.features.shaderResourceMinLod
-                || !required.core.features.shaderResourceMinLod)
-        && (m_deviceFeatures.core.features.sparseBinding
-                || !required.core.features.sparseBinding)
-        && (m_deviceFeatures.core.features.sparseResidencyBuffer
-                || !required.core.features.sparseResidencyBuffer)
-        && (m_deviceFeatures.core.features.sparseResidencyImage2D
-                || !required.core.features.sparseResidencyImage2D)
-        && (m_deviceFeatures.core.features.sparseResidencyImage3D
-                || !required.core.features.sparseResidencyImage3D)
-        && (m_deviceFeatures.core.features.sparseResidency2Samples
-                || !required.core.features.sparseResidency2Samples)
-        && (m_deviceFeatures.core.features.sparseResidency4Samples
-                || !required.core.features.sparseResidency4Samples)
-        && (m_deviceFeatures.core.features.sparseResidency8Samples
-                || !required.core.features.sparseResidency8Samples)
-        && (m_deviceFeatures.core.features.sparseResidency16Samples
-                || !required.core.features.sparseResidency16Samples)
-        && (m_deviceFeatures.core.features.sparseResidencyAliased
-                || !required.core.features.sparseResidencyAliased)
-        && (m_deviceFeatures.core.features.variableMultisampleRate
-                || !required.core.features.variableMultisampleRate)
-        && (m_deviceFeatures.core.features.inheritedQueries
-                || !required.core.features.inheritedQueries)
-        && (m_deviceFeatures.vk11.shaderDrawParameters
-                || !required.vk11.shaderDrawParameters)
-        && (m_deviceFeatures.vk12.samplerMirrorClampToEdge
-                || !required.vk12.samplerMirrorClampToEdge)
-        && (m_deviceFeatures.vk12.drawIndirectCount
-                || !required.vk12.drawIndirectCount)
-        && (m_deviceFeatures.vk12.hostQueryReset
-                || !required.vk12.hostQueryReset)
-        && (m_deviceFeatures.vk12.timelineSemaphore
-                || !required.vk12.timelineSemaphore)
-        && (m_deviceFeatures.vk12.bufferDeviceAddress
-                || !required.vk12.bufferDeviceAddress)
-        && (m_deviceFeatures.vk12.shaderOutputViewportIndex
-                || !required.vk12.shaderOutputViewportIndex)
-        && (m_deviceFeatures.vk12.shaderOutputLayer
-                || !required.vk12.shaderOutputLayer)
-        && (m_deviceFeatures.vk13.pipelineCreationCacheControl
-                || !required.vk13.pipelineCreationCacheControl)
-        && (m_deviceFeatures.vk13.shaderDemoteToHelperInvocation
-                || !required.vk13.shaderDemoteToHelperInvocation)
-        && (m_deviceFeatures.vk13.shaderZeroInitializeWorkgroupMemory
-                || !required.vk13.shaderZeroInitializeWorkgroupMemory)
-        && (m_deviceFeatures.vk13.synchronization2
-                || !required.vk13.synchronization2)
-        && (m_deviceFeatures.vk13.dynamicRendering
-                || !required.vk13.dynamicRendering)
-        && (m_deviceFeatures.vk13.maintenance4
-                || !required.vk13.maintenance4)
-        && (m_deviceFeatures.extAttachmentFeedbackLoopLayout.attachmentFeedbackLoopLayout
-                || !required.extAttachmentFeedbackLoopLayout.attachmentFeedbackLoopLayout)
-        && (m_deviceFeatures.extConservativeRasterization
-                || !required.extConservativeRasterization)
-        && (m_deviceFeatures.extCustomBorderColor.customBorderColors
-                || !required.extCustomBorderColor.customBorderColors)
-        && (m_deviceFeatures.extCustomBorderColor.customBorderColorWithoutFormat
-                || !required.extCustomBorderColor.customBorderColorWithoutFormat)
-        && (m_deviceFeatures.extDepthClipEnable.depthClipEnable
-                || !required.extDepthClipEnable.depthClipEnable)
-        && (m_deviceFeatures.extGraphicsPipelineLibrary.graphicsPipelineLibrary
-                || !required.extGraphicsPipelineLibrary.graphicsPipelineLibrary)
-        && (m_deviceFeatures.extMemoryBudget
-                || !required.extMemoryBudget)
-        && (m_deviceFeatures.extMemoryPriority.memoryPriority
-                || !required.extMemoryPriority.memoryPriority)
-        && (m_deviceFeatures.extNonSeamlessCubeMap.nonSeamlessCubeMap
-                || !required.extNonSeamlessCubeMap.nonSeamlessCubeMap)
-        && (m_deviceFeatures.extRobustness2.robustBufferAccess2
-                || !required.extRobustness2.robustBufferAccess2)
-        && (m_deviceFeatures.extRobustness2.robustImageAccess2
-                || !required.extRobustness2.robustImageAccess2)
-        && (m_deviceFeatures.extRobustness2.nullDescriptor
-                || !required.extRobustness2.nullDescriptor)
-        && (m_deviceFeatures.extShaderModuleIdentifier.shaderModuleIdentifier
-                || !required.extShaderModuleIdentifier.shaderModuleIdentifier)
-        && (m_deviceFeatures.extShaderStencilExport
-                || !required.extShaderStencilExport)
-        && (m_deviceFeatures.extShaderStencilExport
-                || !required.extSwapchainColorSpace)
-        && (m_deviceFeatures.extHdrMetadata
-                || !required.extHdrMetadata)
-        && (m_deviceFeatures.extTransformFeedback.transformFeedback
-                || !required.extTransformFeedback.transformFeedback)
-        && (m_deviceFeatures.extVertexAttributeDivisor.vertexAttributeInstanceRateDivisor
-                || !required.extVertexAttributeDivisor.vertexAttributeInstanceRateDivisor)
-        && (m_deviceFeatures.extVertexAttributeDivisor.vertexAttributeInstanceRateZeroDivisor
-                || !required.extVertexAttributeDivisor.vertexAttributeInstanceRateZeroDivisor);
+    return CHECK_FEATURE_NEED(core.features.robustBufferAccess)
+        && CHECK_FEATURE_NEED(core.features.fullDrawIndexUint32)
+        && CHECK_FEATURE_NEED(core.features.imageCubeArray)
+        && CHECK_FEATURE_NEED(core.features.independentBlend)
+        && CHECK_FEATURE_NEED(core.features.geometryShader)
+        && CHECK_FEATURE_NEED(core.features.tessellationShader)
+        && CHECK_FEATURE_NEED(core.features.sampleRateShading)
+        && CHECK_FEATURE_NEED(core.features.dualSrcBlend)
+        && CHECK_FEATURE_NEED(core.features.logicOp)
+        && CHECK_FEATURE_NEED(core.features.multiDrawIndirect)
+        && CHECK_FEATURE_NEED(core.features.drawIndirectFirstInstance)
+        && CHECK_FEATURE_NEED(core.features.depthClamp)
+        && CHECK_FEATURE_NEED(core.features.depthBiasClamp)
+        && CHECK_FEATURE_NEED(core.features.fillModeNonSolid)
+        && CHECK_FEATURE_NEED(core.features.depthBounds)
+        && CHECK_FEATURE_NEED(core.features.wideLines)
+        && CHECK_FEATURE_NEED(core.features.largePoints)
+        && CHECK_FEATURE_NEED(core.features.alphaToOne)
+        && CHECK_FEATURE_NEED(core.features.multiViewport)
+        && CHECK_FEATURE_NEED(core.features.samplerAnisotropy)
+        && CHECK_FEATURE_NEED(core.features.textureCompressionETC2)
+        && CHECK_FEATURE_NEED(core.features.textureCompressionASTC_LDR)
+        && CHECK_FEATURE_NEED(core.features.textureCompressionBC)
+        && CHECK_FEATURE_NEED(core.features.occlusionQueryPrecise)
+        && CHECK_FEATURE_NEED(core.features.pipelineStatisticsQuery)
+        && CHECK_FEATURE_NEED(core.features.vertexPipelineStoresAndAtomics)
+        && CHECK_FEATURE_NEED(core.features.fragmentStoresAndAtomics)
+        && CHECK_FEATURE_NEED(core.features.shaderTessellationAndGeometryPointSize)
+        && CHECK_FEATURE_NEED(core.features.shaderImageGatherExtended)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageImageExtendedFormats)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageImageMultisample)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageImageReadWithoutFormat)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageImageWriteWithoutFormat)
+        && CHECK_FEATURE_NEED(core.features.shaderUniformBufferArrayDynamicIndexing)
+        && CHECK_FEATURE_NEED(core.features.shaderSampledImageArrayDynamicIndexing)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageBufferArrayDynamicIndexing)
+        && CHECK_FEATURE_NEED(core.features.shaderStorageImageArrayDynamicIndexing)
+        && CHECK_FEATURE_NEED(core.features.shaderClipDistance)
+        && CHECK_FEATURE_NEED(core.features.shaderCullDistance)
+        && CHECK_FEATURE_NEED(core.features.shaderFloat64)
+        && CHECK_FEATURE_NEED(core.features.shaderInt64)
+        && CHECK_FEATURE_NEED(core.features.shaderInt16)
+        && CHECK_FEATURE_NEED(core.features.shaderResourceResidency)
+        && CHECK_FEATURE_NEED(core.features.shaderResourceMinLod)
+        && CHECK_FEATURE_NEED(core.features.sparseBinding)
+        && CHECK_FEATURE_NEED(core.features.sparseResidencyBuffer)
+        && CHECK_FEATURE_NEED(core.features.sparseResidencyImage2D)
+        && CHECK_FEATURE_NEED(core.features.sparseResidencyImage3D)
+        && CHECK_FEATURE_NEED(core.features.sparseResidency2Samples)
+        && CHECK_FEATURE_NEED(core.features.sparseResidency4Samples)
+        && CHECK_FEATURE_NEED(core.features.sparseResidency8Samples)
+        && CHECK_FEATURE_NEED(core.features.sparseResidency16Samples)
+        && CHECK_FEATURE_NEED(core.features.sparseResidencyAliased)
+        && CHECK_FEATURE_NEED(core.features.variableMultisampleRate)
+        && CHECK_FEATURE_NEED(core.features.inheritedQueries)
+        && CHECK_FEATURE_NEED(vk11.shaderDrawParameters)
+        && CHECK_FEATURE_NEED(vk12.samplerMirrorClampToEdge)
+        && CHECK_FEATURE_NEED(vk12.drawIndirectCount)
+        && CHECK_FEATURE_NEED(vk12.hostQueryReset)
+        && CHECK_FEATURE_NEED(vk12.timelineSemaphore)
+        && CHECK_FEATURE_NEED(vk12.bufferDeviceAddress)
+        && CHECK_FEATURE_NEED(vk12.shaderOutputViewportIndex)
+        && CHECK_FEATURE_NEED(vk12.shaderOutputLayer)
+        && CHECK_FEATURE_NEED(vk13.pipelineCreationCacheControl)
+        && CHECK_FEATURE_NEED(vk13.shaderDemoteToHelperInvocation)
+        && CHECK_FEATURE_NEED(vk13.shaderZeroInitializeWorkgroupMemory)
+        && CHECK_FEATURE_NEED(vk13.synchronization2)
+        && CHECK_FEATURE_NEED(vk13.dynamicRendering)
+        && CHECK_FEATURE_NEED(vk13.maintenance4)
+        && CHECK_FEATURE_NEED(extAttachmentFeedbackLoopLayout.attachmentFeedbackLoopLayout)
+        && CHECK_FEATURE_NEED(extConservativeRasterization)
+        && CHECK_FEATURE_NEED(extCustomBorderColor.customBorderColors)
+        && CHECK_FEATURE_NEED(extCustomBorderColor.customBorderColorWithoutFormat)
+        && CHECK_FEATURE_NEED(extDepthClipEnable.depthClipEnable)
+        && CHECK_FEATURE_NEED(extDepthBiasControl.depthBiasControl)
+        && CHECK_FEATURE_NEED(extDepthBiasControl.leastRepresentableValueForceUnormRepresentation)
+        && CHECK_FEATURE_NEED(extDepthBiasControl.floatRepresentation)
+        && CHECK_FEATURE_NEED(extDepthBiasControl.depthBiasExact)
+        && CHECK_FEATURE_NEED(extGraphicsPipelineLibrary.graphicsPipelineLibrary)
+        && CHECK_FEATURE_NEED(extMemoryBudget)
+        && CHECK_FEATURE_NEED(extMemoryPriority.memoryPriority)
+        && CHECK_FEATURE_NEED(extNonSeamlessCubeMap.nonSeamlessCubeMap)
+        && CHECK_FEATURE_NEED(extRobustness2.robustBufferAccess2)
+        && CHECK_FEATURE_NEED(extRobustness2.robustImageAccess2)
+        && CHECK_FEATURE_NEED(extRobustness2.nullDescriptor)
+        && CHECK_FEATURE_NEED(extShaderModuleIdentifier.shaderModuleIdentifier)
+        && CHECK_FEATURE_NEED(extShaderStencilExport)
+        && CHECK_FEATURE_NEED(extSwapchainColorSpace)
+        && CHECK_FEATURE_NEED(extSwapchainMaintenance1.swapchainMaintenance1)
+        && CHECK_FEATURE_NEED(extHdrMetadata)
+        && CHECK_FEATURE_NEED(extTransformFeedback.transformFeedback)
+        && CHECK_FEATURE_NEED(extVertexAttributeDivisor.vertexAttributeInstanceRateDivisor)
+        && CHECK_FEATURE_NEED(extVertexAttributeDivisor.vertexAttributeInstanceRateZeroDivisor);
   }
-  
-  
+
+#undef CHECK_FEATURE_NEED
+
   void DxvkAdapter::enableExtensions(const DxvkNameSet& extensions) {
     m_extraExtensions.merge(extensions);
   }
@@ -439,12 +360,24 @@ namespace dxvk {
       m_deviceFeatures.extExtendedDynamicState3.extendedDynamicState3RasterizationSamples;
     enabledFeatures.extExtendedDynamicState3.extendedDynamicState3SampleMask =
       m_deviceFeatures.extExtendedDynamicState3.extendedDynamicState3SampleMask;
+    enabledFeatures.extExtendedDynamicState3.extendedDynamicState3LineRasterizationMode =
+      m_deviceFeatures.extExtendedDynamicState3.extendedDynamicState3LineRasterizationMode;
 
     // Used for both pNext shader module info, and fast-linking pipelines provided
     // that graphicsPipelineLibraryIndependentInterpolationDecoration is supported
     enabledFeatures.extGraphicsPipelineLibrary.graphicsPipelineLibrary =
       m_deviceFeatures.extGraphicsPipelineLibrary.graphicsPipelineLibrary;
 
+    // Only enable non-default line rasterization features if at least wide lines
+    // and rectangular lines are supported. This saves us several feature checks
+    // in the actual code.
+    if (m_deviceFeatures.core.features.wideLines && m_deviceFeatures.extLineRasterization.rectangularLines) {
+      enabledFeatures.core.features.wideLines = VK_TRUE;
+      enabledFeatures.extLineRasterization.rectangularLines = VK_TRUE;
+      enabledFeatures.extLineRasterization.smoothLines =
+        m_deviceFeatures.extLineRasterization.smoothLines;
+    }
+
     // Enable memory priority if supported to improve memory management
     enabledFeatures.extMemoryPriority.memoryPriority =
       m_deviceFeatures.extMemoryPriority.memoryPriority;
@@ -460,6 +393,28 @@ namespace dxvk {
     enabledFeatures.extShaderModuleIdentifier.shaderModuleIdentifier =
       m_deviceFeatures.extShaderModuleIdentifier.shaderModuleIdentifier;
 
+    // Enable swap chain features that are transparent tot he device
+    enabledFeatures.extSwapchainMaintenance1.swapchainMaintenance1 =
+      m_deviceFeatures.extSwapchainMaintenance1.swapchainMaintenance1 &&
+      instance->extensions().extSurfaceMaintenance1;
+
+    // Enable maintenance5 if supported
+    enabledFeatures.khrMaintenance5.maintenance5 =
+      m_deviceFeatures.khrMaintenance5.maintenance5;
+
+    // Enable present id and present wait together, if possible
+    enabledFeatures.khrPresentId.presentId =
+      m_deviceFeatures.khrPresentId.presentId;
+    enabledFeatures.khrPresentWait.presentWait =
+      m_deviceFeatures.khrPresentId.presentId &&
+      m_deviceFeatures.khrPresentWait.presentWait;
+
+    // Unless we're on an Nvidia driver where these extensions are known to be broken
+    if (matchesDriver(VK_DRIVER_ID_NVIDIA_PROPRIETARY, 0, VK_MAKE_VERSION(535, 0, 0))) {
+      enabledFeatures.khrPresentId.presentId = VK_FALSE;
+      enabledFeatures.khrPresentWait.presentWait = VK_FALSE;
+    }
+
     // Create pNext chain for additional device features
     initFeatureChain(enabledFeatures, devExtensions, instance->extensions());
 
@@ -591,6 +546,10 @@ namespace dxvk {
           enabledFeatures.extDepthClipEnable = *reinterpret_cast<const VkPhysicalDeviceDepthClipEnableFeaturesEXT*>(f);
           break;
 
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_BIAS_CONTROL_FEATURES_EXT:
+          enabledFeatures.extDepthBiasControl = *reinterpret_cast<const VkPhysicalDeviceDepthBiasControlFeaturesEXT*>(f);
+          break;
+
         case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT:
           enabledFeatures.extExtendedDynamicState3 = *reinterpret_cast<const VkPhysicalDeviceExtendedDynamicState3FeaturesEXT*>(f);
           break;
@@ -603,6 +562,10 @@ namespace dxvk {
           enabledFeatures.extGraphicsPipelineLibrary = *reinterpret_cast<const VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT*>(f);
           break;
 
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_EXT:
+          enabledFeatures.extLineRasterization = *reinterpret_cast<const VkPhysicalDeviceLineRasterizationFeaturesEXT*>(f);
+          break;
+
         case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT:
           enabledFeatures.extMemoryPriority = *reinterpret_cast<const VkPhysicalDeviceMemoryPriorityFeaturesEXT*>(f);
           break;
@@ -619,6 +582,10 @@ namespace dxvk {
           enabledFeatures.extShaderModuleIdentifier = *reinterpret_cast<const VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT*>(f);
           break;
 
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SWAPCHAIN_MAINTENANCE_1_FEATURES_EXT:
+          enabledFeatures.extSwapchainMaintenance1 = *reinterpret_cast<const VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT*>(f);
+          break;
+
         case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT:
           enabledFeatures.extTransformFeedback = *reinterpret_cast<const VkPhysicalDeviceTransformFeedbackFeaturesEXT*>(f);
           break;
@@ -627,6 +594,18 @@ namespace dxvk {
           enabledFeatures.extVertexAttributeDivisor = *reinterpret_cast<const VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT*>(f);
           break;
 
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_FEATURES_KHR:
+          enabledFeatures.khrMaintenance5 = *reinterpret_cast<const VkPhysicalDeviceMaintenance5FeaturesKHR*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_ID_FEATURES_KHR:
+          enabledFeatures.khrPresentId = *reinterpret_cast<const VkPhysicalDevicePresentIdFeaturesKHR*>(f);
+          break;
+
+        case VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_WAIT_FEATURES_KHR:
+          enabledFeatures.khrPresentWait = *reinterpret_cast<const VkPhysicalDevicePresentWaitFeaturesKHR*>(f);
+          break;
+
         default:
           // Ignore any unknown feature structs
           break;
@@ -769,6 +748,11 @@ namespace dxvk {
       m_deviceInfo.extGraphicsPipelineLibrary.pNext = std::exchange(m_deviceInfo.core.pNext, &m_deviceInfo.extGraphicsPipelineLibrary);
     }
 
+    if (m_deviceExtensions.supports(VK_EXT_LINE_RASTERIZATION_EXTENSION_NAME)) {
+      m_deviceInfo.extLineRasterization.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_PROPERTIES_EXT;
+      m_deviceInfo.extLineRasterization.pNext = std::exchange(m_deviceInfo.core.pNext, &m_deviceInfo.extLineRasterization);
+    }
+
     if (m_deviceExtensions.supports(VK_EXT_ROBUSTNESS_2_EXTENSION_NAME)) {
       m_deviceInfo.extRobustness2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_PROPERTIES_EXT;
       m_deviceInfo.extRobustness2.pNext = std::exchange(m_deviceInfo.core.pNext, &m_deviceInfo.extRobustness2);
@@ -784,6 +768,11 @@ namespace dxvk {
       m_deviceInfo.extVertexAttributeDivisor.pNext = std::exchange(m_deviceInfo.core.pNext, &m_deviceInfo.extVertexAttributeDivisor);
     }
 
+    if (m_deviceExtensions.supports(VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+      m_deviceInfo.khrMaintenance5.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_PROPERTIES_KHR;
+      m_deviceInfo.khrMaintenance5.pNext = std::exchange(m_deviceInfo.core.pNext, &m_deviceInfo.khrMaintenance5);
+    }
+
     // Query full device properties for all enabled extensions
     m_vki->vkGetPhysicalDeviceProperties2(m_handle, &m_deviceInfo.core);
     
@@ -842,6 +831,11 @@ namespace dxvk {
       m_deviceFeatures.extDepthClipEnable.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extDepthClipEnable);
     }
 
+    if (m_deviceExtensions.supports(VK_EXT_DEPTH_BIAS_CONTROL_EXTENSION_NAME)) {
+      m_deviceFeatures.extDepthBiasControl.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_BIAS_CONTROL_FEATURES_EXT;
+      m_deviceFeatures.extDepthBiasControl.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extDepthBiasControl);
+    }
+
     if (m_deviceExtensions.supports(VK_EXT_EXTENDED_DYNAMIC_STATE_3_EXTENSION_NAME)) {
       m_deviceFeatures.extExtendedDynamicState3.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT;
       m_deviceFeatures.extExtendedDynamicState3.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extExtendedDynamicState3);
@@ -860,6 +854,11 @@ namespace dxvk {
       m_deviceFeatures.extGraphicsPipelineLibrary.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extGraphicsPipelineLibrary);
     }
 
+    if (m_deviceExtensions.supports(VK_EXT_LINE_RASTERIZATION_EXTENSION_NAME)) {
+      m_deviceFeatures.extLineRasterization.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_EXT;
+      m_deviceFeatures.extLineRasterization.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extLineRasterization);
+    }
+
     if (m_deviceExtensions.supports(VK_EXT_MEMORY_BUDGET_EXTENSION_NAME))
       m_deviceFeatures.extMemoryBudget = VK_TRUE;
 
@@ -889,6 +888,11 @@ namespace dxvk {
     if (m_deviceExtensions.supports(VK_EXT_SWAPCHAIN_COLOR_SPACE_EXTENSION_NAME))
       m_deviceFeatures.extSwapchainColorSpace = VK_TRUE;
 
+    if (m_deviceExtensions.supports(VK_EXT_SWAPCHAIN_MAINTENANCE_1_EXTENSION_NAME)) {
+      m_deviceFeatures.extSwapchainMaintenance1.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SWAPCHAIN_MAINTENANCE_1_FEATURES_EXT;
+      m_deviceFeatures.extSwapchainMaintenance1.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.extSwapchainMaintenance1);
+    }
+
     if (m_deviceExtensions.supports(VK_EXT_HDR_METADATA_EXTENSION_NAME))
       m_deviceFeatures.extHdrMetadata = VK_TRUE;
 
@@ -908,6 +912,21 @@ namespace dxvk {
     if (m_deviceExtensions.supports(VK_KHR_EXTERNAL_SEMAPHORE_WIN32_EXTENSION_NAME))
       m_deviceFeatures.khrExternalSemaphoreWin32 = VK_TRUE;
 
+    if (m_deviceExtensions.supports(VK_KHR_MAINTENANCE_5_EXTENSION_NAME)) {
+      m_deviceFeatures.khrMaintenance5.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_FEATURES_KHR;
+      m_deviceFeatures.khrMaintenance5.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.khrMaintenance5);
+    }
+
+    if (m_deviceExtensions.supports(VK_KHR_PRESENT_ID_EXTENSION_NAME)) {
+      m_deviceFeatures.khrPresentId.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_ID_FEATURES_KHR;
+      m_deviceFeatures.khrPresentId.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.khrPresentId);
+    }
+
+    if (m_deviceExtensions.supports(VK_KHR_PRESENT_WAIT_EXTENSION_NAME)) {
+      m_deviceFeatures.khrPresentWait.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_WAIT_FEATURES_KHR;
+      m_deviceFeatures.khrPresentWait.pNext = std::exchange(m_deviceFeatures.core.pNext, &m_deviceFeatures.khrPresentWait);
+    }
+
     if (m_deviceExtensions.supports(VK_NVX_BINARY_IMPORT_EXTENSION_NAME))
       m_deviceFeatures.nvxBinaryImport = VK_TRUE;
 
@@ -950,11 +969,13 @@ namespace dxvk {
       &devExtensions.extConservativeRasterization,
       &devExtensions.extCustomBorderColor,
       &devExtensions.extDepthClipEnable,
+      &devExtensions.extDepthBiasControl,
       &devExtensions.extExtendedDynamicState3,
       &devExtensions.extFragmentShaderInterlock,
       &devExtensions.extFullScreenExclusive,
       &devExtensions.extGraphicsPipelineLibrary,
       &devExtensions.extHdrMetadata,
+      &devExtensions.extLineRasterization,
       &devExtensions.extMemoryBudget,
       &devExtensions.extMemoryPriority,
       &devExtensions.extNonSeamlessCubeMap,
@@ -962,12 +983,17 @@ namespace dxvk {
       &devExtensions.extShaderModuleIdentifier,
       &devExtensions.extShaderStencilExport,
       &devExtensions.extSwapchainColorSpace,
+      &devExtensions.extSwapchainMaintenance1,
       &devExtensions.extTransformFeedback,
       &devExtensions.extVertexAttributeDivisor,
       &devExtensions.khrExternalMemoryWin32,
       &devExtensions.khrExternalSemaphoreWin32,
+      &devExtensions.khrMaintenance5,
       &devExtensions.khrPipelineLibrary,
+      &devExtensions.khrPresentId,
+      &devExtensions.khrPresentWait,
       &devExtensions.khrSwapchain,
+      &devExtensions.khrWin32KeyedMutex,
       &devExtensions.nvxBinaryImport,
       &devExtensions.nvxImageViewHandle,
     }};
@@ -1011,6 +1037,11 @@ namespace dxvk {
       enabledFeatures.extDepthClipEnable.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extDepthClipEnable);
     }
 
+    if (devExtensions.extDepthBiasControl) {
+      enabledFeatures.extDepthBiasControl.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_BIAS_CONTROL_FEATURES_EXT;
+      enabledFeatures.extDepthBiasControl.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extDepthBiasControl);
+    }
+
     if (devExtensions.extExtendedDynamicState3) {
       enabledFeatures.extExtendedDynamicState3.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT;
       enabledFeatures.extExtendedDynamicState3.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extExtendedDynamicState3);
@@ -1029,6 +1060,11 @@ namespace dxvk {
       enabledFeatures.extGraphicsPipelineLibrary.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extGraphicsPipelineLibrary);
     }
 
+    if (devExtensions.extLineRasterization) {
+      enabledFeatures.extLineRasterization.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_EXT;
+      enabledFeatures.extLineRasterization.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extLineRasterization);
+    }
+
     if (devExtensions.extMemoryBudget)
       enabledFeatures.extMemoryBudget = VK_TRUE;
 
@@ -1058,6 +1094,11 @@ namespace dxvk {
     if (devExtensions.extSwapchainColorSpace)
       enabledFeatures.extSwapchainColorSpace = VK_TRUE;
 
+    if (devExtensions.extSwapchainMaintenance1) {
+      enabledFeatures.extSwapchainMaintenance1.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SWAPCHAIN_MAINTENANCE_1_FEATURES_EXT;
+      enabledFeatures.extSwapchainMaintenance1.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.extSwapchainMaintenance1);
+    }
+
     if (devExtensions.extHdrMetadata)
       enabledFeatures.extHdrMetadata = VK_TRUE;
 
@@ -1077,11 +1118,29 @@ namespace dxvk {
     if (devExtensions.khrExternalSemaphoreWin32)
       enabledFeatures.khrExternalSemaphoreWin32 = VK_TRUE;
 
+    if (devExtensions.khrMaintenance5) {
+      enabledFeatures.khrMaintenance5.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_FEATURES_KHR;
+      enabledFeatures.khrMaintenance5.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.khrMaintenance5);
+    }
+
+    if (devExtensions.khrPresentId) {
+      enabledFeatures.khrPresentId.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_ID_FEATURES_KHR;
+      enabledFeatures.khrPresentId.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.khrPresentId);
+    }
+
+    if (devExtensions.khrPresentWait) {
+      enabledFeatures.khrPresentWait.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_WAIT_FEATURES_KHR;
+      enabledFeatures.khrPresentWait.pNext = std::exchange(enabledFeatures.core.pNext, &enabledFeatures.khrPresentWait);
+    }
+
     if (devExtensions.nvxBinaryImport)
       enabledFeatures.nvxBinaryImport = VK_TRUE;
 
     if (devExtensions.nvxImageViewHandle)
       enabledFeatures.nvxImageViewHandle = VK_TRUE;
+
+    if (devExtensions.khrWin32KeyedMutex)
+      enabledFeatures.khrWin32KeyedMutex = VK_TRUE;
   }
 
   
@@ -1108,6 +1167,7 @@ namespace dxvk {
       "\n  depthBiasClamp                         : ", features.core.features.depthBiasClamp ? "1" : "0",
       "\n  fillModeNonSolid                       : ", features.core.features.fillModeNonSolid ? "1" : "0",
       "\n  depthBounds                            : ", features.core.features.depthBounds ? "1" : "0",
+      "\n  wideLines                              : ", features.core.features.wideLines ? "1" : "0",
       "\n  multiViewport                          : ", features.core.features.multiViewport ? "1" : "0",
       "\n  samplerAnisotropy                      : ", features.core.features.samplerAnisotropy ? "1" : "0",
       "\n  textureCompressionBC                   : ", features.core.features.textureCompressionBC ? "1" : "0",
@@ -1162,11 +1222,17 @@ namespace dxvk {
       "\n  customBorderColorWithoutFormat         : ", features.extCustomBorderColor.customBorderColorWithoutFormat ? "1" : "0",
       "\n", VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME,
       "\n  depthClipEnable                        : ", features.extDepthClipEnable.depthClipEnable ? "1" : "0",
+      "\n", VK_EXT_DEPTH_BIAS_CONTROL_EXTENSION_NAME,
+      "\n  depthBiasControl                       : ", features.extDepthBiasControl.depthBiasControl ? "1" : "0",
+      "\n  leastRepresentableValueForceUnormRepresentation : ", features.extDepthBiasControl.leastRepresentableValueForceUnormRepresentation ? "1" : "0",
+      "\n  floatRepresentation                    : ", features.extDepthBiasControl.floatRepresentation ? "1" : "0",
+      "\n  depthBiasExact                         : ", features.extDepthBiasControl.depthBiasExact ? "1" : "0",
       "\n", VK_EXT_EXTENDED_DYNAMIC_STATE_3_EXTENSION_NAME,
       "\n  extDynamicState3AlphaToCoverageEnable  : ", features.extExtendedDynamicState3.extendedDynamicState3AlphaToCoverageEnable ? "1" : "0",
       "\n  extDynamicState3DepthClipEnable        : ", features.extExtendedDynamicState3.extendedDynamicState3DepthClipEnable ? "1" : "0",
       "\n  extDynamicState3RasterizationSamples   : ", features.extExtendedDynamicState3.extendedDynamicState3RasterizationSamples ? "1" : "0",
       "\n  extDynamicState3SampleMask             : ", features.extExtendedDynamicState3.extendedDynamicState3SampleMask ? "1" : "0",
+      "\n  extDynamicState3LineRasterizationMode  : ", features.extExtendedDynamicState3.extendedDynamicState3LineRasterizationMode ? "1" : "0",
       "\n", VK_EXT_FRAGMENT_SHADER_INTERLOCK_EXTENSION_NAME,
       "\n  fragmentShaderSampleInterlock          : ", features.extFragmentShaderInterlock.fragmentShaderSampleInterlock ? "1" : "0",
       "\n  fragmentShaderPixelInterlock           : ", features.extFragmentShaderInterlock.fragmentShaderPixelInterlock ? "1" : "0",
@@ -1174,6 +1240,9 @@ namespace dxvk {
       "\n  extension supported                    : ", features.extFullScreenExclusive ? "1" : "0",
       "\n", VK_EXT_GRAPHICS_PIPELINE_LIBRARY_EXTENSION_NAME,
       "\n  graphicsPipelineLibrary                : ", features.extGraphicsPipelineLibrary.graphicsPipelineLibrary ? "1" : "0",
+      "\n", VK_EXT_LINE_RASTERIZATION_EXTENSION_NAME,
+      "\n  rectangularLines                       : ", features.extLineRasterization.rectangularLines ? "1" : "0",
+      "\n  smoothLines                            : ", features.extLineRasterization.smoothLines ? "1" : "0",
       "\n", VK_EXT_MEMORY_BUDGET_EXTENSION_NAME,
       "\n  extension supported                    : ", features.extMemoryBudget ? "1" : "0",
       "\n", VK_EXT_MEMORY_PRIORITY_EXTENSION_NAME,
@@ -1190,6 +1259,8 @@ namespace dxvk {
       "\n  extension supported                    : ", features.extShaderStencilExport ? "1" : "0",
       "\n", VK_EXT_SWAPCHAIN_COLOR_SPACE_EXTENSION_NAME,
       "\n  extension supported                    : ", features.extSwapchainColorSpace ? "1" : "0",
+      "\n", VK_EXT_SWAPCHAIN_MAINTENANCE_1_EXTENSION_NAME,
+      "\n  swapchainMaintenance1                  : ", features.extSwapchainMaintenance1.swapchainMaintenance1 ? "1" : "0",
       "\n", VK_EXT_HDR_METADATA_EXTENSION_NAME,
       "\n  extension supported                    : ", features.extHdrMetadata ? "1" : "0",
       "\n", VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME,
@@ -1202,10 +1273,18 @@ namespace dxvk {
       "\n  extension supported                    : ", features.khrExternalMemoryWin32 ? "1" : "0",
       "\n", VK_KHR_EXTERNAL_SEMAPHORE_WIN32_EXTENSION_NAME,
       "\n  extension supported                    : ", features.khrExternalSemaphoreWin32 ? "1" : "0",
+      "\n", VK_KHR_MAINTENANCE_5_EXTENSION_NAME,
+      "\n  maintenance5                           : ", features.khrMaintenance5.maintenance5 ? "1" : "0",
+      "\n", VK_KHR_PRESENT_ID_EXTENSION_NAME,
+      "\n  presentId                              : ", features.khrPresentId.presentId ? "1" : "0",
+      "\n", VK_KHR_PRESENT_WAIT_EXTENSION_NAME,
+      "\n  presentWait                            : ", features.khrPresentWait.presentWait ? "1" : "0",
       "\n", VK_NVX_BINARY_IMPORT_EXTENSION_NAME,
       "\n  extension supported                    : ", features.nvxBinaryImport ? "1" : "0",
       "\n", VK_NVX_IMAGE_VIEW_HANDLE_EXTENSION_NAME,
-      "\n  extension supported                    : ", features.nvxImageViewHandle ? "1" : "0"));
+      "\n  extension supported                    : ", features.nvxImageViewHandle ? "1" : "0",
+      "\n", VK_KHR_WIN32_KEYED_MUTEX_EXTENSION_NAME,
+      "\n  extension supported                    : ", features.khrWin32KeyedMutex ? "1" : "0"));
   }
 
 
