@@ -183,7 +183,8 @@ namespace dxvk {
 
   DxvkMemoryAllocator::DxvkMemoryAllocator(DxvkDevice* device)
   : m_device          (device),
-    m_memProps        (device->adapter()->memoryProperties()) {
+    m_memProps        (device->adapter()->memoryProperties()),
+    m_maxChunkSize    (determineMaxChunkSize(device)) {
     for (uint32_t i = 0; i < m_memProps.memoryHeapCount; i++) {
       m_memHeaps[i].properties = m_memProps.memoryHeaps[i];
       m_memHeaps[i].stats      = DxvkMemoryStats { 0, 0 };
@@ -365,8 +366,10 @@ namespace dxvk {
         memory = DxvkMemory(this, nullptr, type, devMem.memHandle, 0, size, devMem.memPointer);
     }
 
-    if (memory)
+    if (memory) {
       type->heap->stats.memoryUsed += memory.m_length;
+      m_device->notifyMemoryUse(type->heapId, memory.m_length);
+    }
 
     return memory;
   }
@@ -430,7 +433,7 @@ namespace dxvk {
     }
 
     type->heap->stats.memoryAllocated += size;
-    m_device->adapter()->notifyHeapMemoryAlloc(type->heapId, size);
+    m_device->notifyMemoryAlloc(type->heapId, size);
     return result;
   }
 
@@ -453,6 +456,8 @@ namespace dxvk {
       devMem.memSize    = memory.m_length;
       this->freeDeviceMemory(memory.m_type, devMem);
     }
+
+    m_device->notifyMemoryUse(memory.m_type->heapId, -memory.m_length);
   }
 
   
@@ -484,7 +489,7 @@ namespace dxvk {
     vk->vkFreeMemory(vk->device(), memory.memHandle, nullptr);
 
     type->heap->stats.memoryAllocated -= memory.memSize;
-    m_device->adapter()->notifyHeapMemoryFree(type->heapId, memory.memSize);
+    m_device->notifyMemoryAlloc(type->heapId, memory.memSize);
   }
 
 
@@ -493,15 +498,15 @@ namespace dxvk {
     VkMemoryHeap heap = m_memProps.memoryHeaps[type.heapIndex];
 
     // Default to a chunk size of 256 MiB
-    VkDeviceSize chunkSize = 256 << 20;
+    VkDeviceSize chunkSize = m_maxChunkSize;
 
     if (hints.test(DxvkMemoryFlag::Small))
-      chunkSize = 16 << 20;
+      chunkSize = std::min<VkDeviceSize>(chunkSize, 16 << 20);
 
     // Try to waste a bit less system memory especially in
     // 32-bit applications due to address space constraints
     if (type.propertyFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
-      chunkSize = (env::is32BitHostPlatform() ? 16 : 64) << 20;
+      chunkSize = std::min<VkDeviceSize>((env::is32BitHostPlatform() ? 16 : 64) << 20, chunkSize);
 
     // Reduce the chunk size on small heaps so
     // we can at least fit in 15 allocations
@@ -638,6 +643,17 @@ namespace dxvk {
   }
 
 
+  VkDeviceSize DxvkMemoryAllocator::determineMaxChunkSize(
+          DxvkDevice*           device) const {
+    int32_t option = device->config().maxChunkSize;
+
+    if (option <= 0)
+      option = 256;
+
+    return VkDeviceSize(option) << 20;
+  }
+
+
   void DxvkMemoryAllocator::logMemoryError(const VkMemoryRequirements& req) const {
     std::stringstream sstr;
     sstr << "DxvkMemoryAllocator: Memory allocation failed" << std::endl
