@@ -32,6 +32,10 @@ namespace dxvk {
 
     /// HUD elements
     std::string hud;
+
+    /// Forces swap chain into MAILBOX (if true)
+    /// or FIFO_RELAXED (if false) present mode
+    Tristate tearFree;
   };
 
 }
