@@ -6,9 +6,8 @@
 
 #include "../util/thread.h"
 
-#include "../vulkan/vulkan_presenter.h"
-
 #include "dxvk_cmdlist.h"
+#include "dxvk_presenter.h"
 
 namespace dxvk {
   
@@ -43,7 +42,9 @@ namespace dxvk {
    * a swap chain image on the device.
    */
   struct DxvkPresentInfo {
-    Rc<vk::Presenter>   presenter;
+    Rc<Presenter>       presenter;
+    VkPresentModeKHR    presentMode;
+    uint64_t            frameId;
   };
 
 
@@ -51,6 +52,7 @@ namespace dxvk {
    * \brief Submission queue entry
    */
   struct DxvkSubmitEntry {
+    VkResult            result;
     DxvkSubmitStatus*   status;
     DxvkSubmitInfo      submit;
     DxvkPresentInfo     present;
@@ -70,17 +72,6 @@ namespace dxvk {
 
     ~DxvkSubmissionQueue();
 
-    /**
-     * \brief Number of pending submissions
-     * 
-     * A return value of 0 indicates
-     * that the GPU is currently idle.
-     * \returns Pending submission count
-     */
-    uint32_t pendingSubmissions() const {
-      return m_pending.load();
-    }
-
     /**
      * \brief Retrieves estimated GPU idle time
      *
@@ -160,6 +151,11 @@ namespace dxvk {
       m_finishCond.wait(lock, pred);
     }
 
+    /**
+     * \brief Waits for all submissions to complete
+     */
+    void waitForIdle();
+
     /**
      * \brief Locks device queue
      *
@@ -186,7 +182,6 @@ namespace dxvk {
     std::atomic<VkResult>       m_lastError = { VK_SUCCESS };
     
     std::atomic<bool>           m_stopped = { false };
-    std::atomic<uint32_t>       m_pending = { 0u };
     std::atomic<uint64_t>       m_gpuIdle = { 0ull };
 
     dxvk::mutex                 m_mutex;
