@@ -37,7 +37,7 @@ namespace dxvk {
     { R"(\\evelauncher\.exe$)", {{
       { "d3d11.maxFeatureLevel",            "12_1" },
     }} },
-    /* The Evil Within: Submits command lists     * 
+    /* The Evil Within: Submits command lists     *
      * multiple times                             */
     { R"(\\EvilWithin(Demo)?\.exe$)", {{
       { "d3d11.dcSingleUseMode",            "False" },
@@ -49,18 +49,18 @@ namespace dxvk {
      * hardware.                                  */
     { R"(\\(FarCry2|farcry2game)\.exe$)", {{
       { "d3d9.customVendorId",              "10de" },
-      { "d3d9.apitraceMode",                "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* Far Cry 3: Assumes clear(0.5) on an UNORM  *
      * format to result in 128 on AMD and 127 on  *
      * Nvidia. We assume that the Vulkan drivers  *
      * match the clear behaviour of D3D11.        */
     { R"(\\(farcry3|fc3_blooddragon)_d3d11\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Far Cry 4: Same as Far Cry 3               */
     { R"(\\FarCry4\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Frostpunk: Renders one frame with D3D9     *
      * after creating the DXGI swap chain         */
@@ -103,15 +103,15 @@ namespace dxvk {
     }} },
     /* Call of Duty WW2                           */
     { R"(\\s2_sp64_ship\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Need for Speed 2015                        */
     { R"(\\NFS16\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Mass Effect Andromeda                      */
     { R"(\\MassEffectAndromeda\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Mirror`s Edge Catalyst: Crashes on AMD     */
     { R"(\\MirrorsEdgeCatalyst(Trial)?\.exe$)", {{
@@ -119,7 +119,7 @@ namespace dxvk {
     }} },
     /* Star Wars Battlefront (2015)               */
     { R"(\\starwarsbattlefront(trial)?\.exe$)", {{
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.customVendorId",              "10de" },
     }} },
     /* NieR Replicant                             */
     { R"(\\NieR Replicant ver\.1\.22474487139\.exe)", {{
@@ -141,7 +141,7 @@ namespace dxvk {
     }} },
     /* Modern Warfare 2 Campaign Remastered       *
      * AMD AGS crash same as above                */
-    { R"(\\MW2CR.exe\.exe$)", {{
+    { R"(\\MW2CR\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
     }} },
     /* Crysis 3 - slower if it notices AMD card     *
@@ -265,7 +265,7 @@ namespace dxvk {
     { R"(\\GoW\.exe$)", {{
       { "d3d11.ignoreGraphicsBarriers",     "True" },
       { "d3d11.relaxedBarriers",            "True" },
-      { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
       { "dxgi.maxFrameLatency",             "1"    },
     }} },
     /* AoE 2 DE - runs poorly for some users      */
@@ -290,12 +290,12 @@ namespace dxvk {
     }} },
     /* A Way Out: fix for stuttering and low fps  */
     { R"(\\AWayOut(_friend)?\.exe$)", {{
-      { "dxgi.maxFrameLatency",                "1" },
+      { "dxgi.maxFrameLatency",             "1" },
     }} },
     /* Garden Warfare 2
        Won't start on amd Id without atiadlxx     */
-    { R"(\\GW2.Main_Win64_Retail\.exe$)", {{
-      { "dxgi.customVendorId",           "10de"   },
+    { R"(\\GW2\.Main_Win64_Retail\.exe$)", {{
+      { "dxgi.customVendorId",              "10de"   },
     }} },
     /* DayZ */
     { R"(\\DayZ_x64\.exe$)", {{
@@ -310,27 +310,27 @@ namespace dxvk {
     /* Metal Gear Solid V: Ground Zeroes          *
      * Texture quality can break at high vram     */
     { R"(\\MgsGroundZeroes\.exe$)", {{
-      { "dxgi.maxDeviceMemory",     "4095" },
+      { "dxgi.maxDeviceMemory",             "4095" },
     }} },
     /* Shantae and the Pirate's Curse             *
      * Game speeds up above 60 fps                */
     { R"(\\ShantaeCurse\.exe$)", {{
-      { "dxgi.maxFrameRate",                 "60" },
+      { "dxgi.maxFrameRate",                "60" },
     }} },
     /* Mighty Switch Force! Collection            *
      * Games speed up above 60 fps                */
     { R"(\\MSFC\.exe$)", {{
-      { "dxgi.maxFrameRate",                 "60" },
+      { "dxgi.maxFrameRate",                "60" },
     }} },
     /* Cardfight!! Vanguard Dear Days:            *
      * Submits command lists multiple times       */
     { R"(\\VG2\.exe$)", {{
-      { "d3d11.dcSingleUseMode",             "False" },
+      { "d3d11.dcSingleUseMode",            "False" },
     }} },
     /* Battlefield: Bad Company 2                 *
      * Gets rid of black flickering               */
     { R"(\\BFBC2Game\.exe$)", {{
-      { "d3d11.floatControls",            "False" },
+      { "d3d11.floatControls",              "False" },
     }} },
     /* Sonic Frontiers - flickering shadows and   *
      * vegetation when GPU-bound                  */
@@ -345,11 +345,11 @@ namespace dxvk {
     /* SpellForce 3 Reforced & expansions         *
      * Greatly improves CPU bound performance     */
     { R"(\\SF3ClientFinal\.exe$)", {{
-      { "d3d11.cachedDynamicResources",        "v" },
+      { "d3d11.cachedDynamicResources",     "v" },
     }} },
     /* Tom Clancy's Ghost Recon Breakpoint        */
     { R"(\\GRB\.exe$)", {{
-      { "dxgi.nvapiHack",                  "False" },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* GTA V performance issues                   */
     { R"(\\GTA5\.exe$)", {{
@@ -360,6 +360,63 @@ namespace dxvk {
     { R"(\\CrashBandicootNSaneTrilogy\.exe$)", {{
       { "dxgi.syncInterval",                "1"   },
     }} },
+    /* SnowRunner                                 */
+    { R"(\\SnowRunner\.exe$)", {{
+      { "d3d11.dcSingleUseMode",            "False" },
+    }} },
+    /* Rockstar Games Launcher                    */
+    { R"(\\Rockstar Games\\Launcher\\Launcher\.exe$)", {{
+      { "dxvk.maxChunkSize",                "1"   },
+    }} },
+    /* Rockstar Social Club                       */
+    { R"(\\Rockstar Games\\Social Club\\SocialClubHelper\.exe$)", {{
+      { "dxvk.maxChunkSize",                "1"   },
+    }} },
+    /* EA Desktop App                             */
+    { R"(\\EADesktop\.exe$)", {{
+      { "dxvk.maxChunkSize",                "1"   },
+    }} },
+    /* GOG Galaxy                                 */
+    { R"(\\GalaxyClient\.exe$)", {{
+      { "dxvk.maxChunkSize",                "1"   },
+    }} },
+    /* Fallout 76
+     * Game tries to be too "smart" and changes sync
+     * interval based on performance (in fullscreen)
+     * or tries to match (or ratio below) 60fps
+     * (in windowed).
+     *
+     * Ends up getting in a loop where it will switch
+     * and start stuttering, or get stuck at targeting
+     * 30Hz in fullscreen.
+     * Windowed mode being locked to 60fps as well is
+     * pretty suboptimal...
+     */
+    { R"(\\Fallout76\.exe$)", {{
+      { "dxgi.syncInterval",                "1" },
+    }} },
+    /* Blizzard Entertainment Battle.net          */
+    { R"(\\Battle\.net\.exe$)", {{
+      { "dxvk.maxChunkSize",                "1" },
+    }} },
+    /* Bladestorm Nightmare                       *
+     * Game speed increases when above 60 fps in  *
+     * the tavern area                            */
+    { R"(\\BLADESTORM Nightmare\\Launch_(EA|JP)\.exe$)", {{
+      { "dxgi.maxFrameRate",                "60"  },
+    }} },
+    /* Ghost Recon Wildlands                      */
+    { R"(\\GRW\.exe$)", {{
+      { "d3d11.dcSingleUseMode",            "False" },
+    }} },
+    /* Vindictus d3d11 CPU bound perf             */
+    { R"(\\Vindictus(_x64)?\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "cr"   },
+    }} },
+    /* Riders Republic - Statically linked AMDAGS */
+    { R"(\\RidersRepublic(_BE)?\.exe$)", {{
+      { "dxgi.hideAmdGpu",                "True"   },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -407,7 +464,7 @@ namespace dxvk {
       { "d3d9.memoryTrackTest",             "True" },
     }} },
     /* Dead Space uses the a NULL render target instead
-       of a 1x1 one if DF24 is NOT supported     
+       of a 1x1 one if DF24 is NOT supported
        Mouse and physics issues above 60 FPS
        Built-in Vsync Locks the game to 30 FPS    */
     { R"(\\Dead Space\.exe$)", {{
@@ -462,10 +519,6 @@ namespace dxvk {
     { R"(\\Neptunia\.exe$)", {{
       { "d3d9.forceAspectRatio",            "16:9" },
     }} },
-    /* D&D - The Temple Of Elemental Evil          */
-    { R"(\\ToEE(a)?\.exe$)", {{
-      { "d3d9.allowDiscard",                "False" },
-    }} },
     /* GTA IV (NVAPI)                             */
     /* Also thinks we're always on Intel          *
      * and will report/use bad amounts of VRAM.
@@ -477,7 +530,7 @@ namespace dxvk {
       { "d3d9.customVendorId",              "1002" },
       { "dxgi.emulateUMA",                  "True" },
       { "d3d9.supportDFFormats",            "False" },
-      { "d3d9.deviceLost",                  "True" },
+      { "d3d9.deviceLostOnFocusLoss",       "True" },
     }} },
     /* Battlefield 2 (bad z-pass)                 */
     { R"(\\BF2\.exe$)", {{
@@ -487,13 +540,17 @@ namespace dxvk {
     { R"(\\SpellForce2.*\.exe$)", {{
       { "d3d9.forceSamplerTypeSpecConstants", "True" },
     }} },
-    /* Tomb Raider: Legend                       */
-    { R"(\\trl\.exe$)", {{
-      { "d3d9.apitraceMode",                "True" },
+    /* Tomb Raider: Legend, Anniversary, Underworld  *
+     * Read from a buffer created with:              *
+     * D3DPOOL_DEFAULT,                              *
+     * D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY buffer  */
+    { R"(\\(trl|tra|tru)\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+      { "d3d9.maxFrameRate",                "60" },
     }} },
     /* Everquest                                 */
     { R"(\\eqgame\.exe$)", {{
-      { "d3d9.apitraceMode",                "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* Dark Messiah of Might & Magic             */
     { R"(\\mm\.exe$)", {{
@@ -573,10 +630,7 @@ namespace dxvk {
     /* BlazBlue Centralfiction                  */
     { R"(\\BBCF\.exe$)", {{
       { "d3d9.floatEmulation",              "Strict" },
-    }} },
-    /* Resident Evil games using MT Framework   */
-    { R"(\\(rerev|re0hd|bhd)\.exe$)", {{
-      { "d3d9.allowDirectBufferMapping",                "False" },
+      { "d3d9.textureMemory",               "0"   },
     }} },
     /* Limbo                                    */
     { R"(\\limbo\.exe$)", {{
@@ -606,7 +660,7 @@ namespace dxvk {
       { "d3d9.memoryTrackTest",             "True" },
       { "d3d9.maxAvailableMemory",          "2048" },
     }} },
-    /* Myst V End of Ages                             
+    /* Myst V End of Ages
        Game has white textures on amd radv.
        Expects Nvidia, Intel or ATI VendorId.
        "Radeon" in gpu description also works   */
@@ -615,20 +669,20 @@ namespace dxvk {
     }} },
     /* Supreme Commander & Forged Alliance Forever */
     { R"(\\(SupremeCommander|ForgedAlliance)\.exe$)", {{
-      { "d3d9.floatEmulation",            "Strict" },
+      { "d3d9.floatEmulation",              "Strict" },
     }} },
     /* Star Wars The Old Republic */
     { R"(\\swtor\.exe$)", {{
       { "d3d9.forceSamplerTypeSpecConstants", "True" },
     }} },
-    /* Bionic Commando                          
+    /* Bionic Commando
        Physics break at high fps               */
     { R"(\\bionic_commando\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
     /* Need For Speed 3 modern patch            */
     { R"(\\nfs3\.exe$)", {{
-      { "d3d9.enableDialogMode",          "True" },
+      { "d3d9.enableDialogMode",            "True" },
     }} },
     /* Beyond Good And Evil                     *
      * UI breaks at high fps                     */
@@ -637,21 +691,21 @@ namespace dxvk {
     }} },
     /* King Of Fighters XIII                     *
      * In-game speed increases on high FPS       */
-    { R"(\\kofxiii\.exe$)", {{
+    { R"(\\kof(xiii|13_win32_Release)\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
     /* YS Origin                                *
      * Helps very bad frametimes in some areas  */
     { R"(\\yso_win\.exe$)", {{
-      { "d3d9.maxFrameLatency",              "1" },
+      { "d3d9.maxFrameLatency",             "1" },
     }} },
     /* Saints Row 2 - Prevents unmap crash      */
     { R"(\\SR2_pc\.exe$)", {{
-      { "d3d9.textureMemory",                "0" },
+      { "d3d9.textureMemory",               "0" },
     }} },
     /* Witcher 1: Very long loading times       */
     { R"(\\witcher\.exe$)", {{
-      { "d3d9.apitraceMode",              "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* Guitar Hero World Tour                   *
      * Very prone to address space crashes      */
@@ -676,7 +730,7 @@ namespace dxvk {
     }} },
     /* Hammer World Editor                      */
     { R"(\\(hammer(plusplus)?|mallet|wc)\.exe$)", {{
-      { "d3d9.apitraceMode",                "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* Dragon Age Origins                       *
      * Keeps unmapping the same 3 1MB buffers   *
@@ -693,7 +747,7 @@ namespace dxvk {
      * Helps performance when Resizable BAR    *
      * is enabled                              */
     { R"(\\ASN_App_PcDx9_Final\.exe$)", {{
-      { "d3d9.apitraceMode",                "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* Black Mesa                              *
      * Artifacts & broken flashlight on Intel  */
@@ -703,30 +757,70 @@ namespace dxvk {
     /* Final Fantasy XIV - Direct3D 9 mode     *
      * Can crash with unmapping                */
     { R"(\\ffxiv\.exe$)", {{
-      { "d3d9.textureMemory",                "0"   },
-    }} },
-    /* Secret World Legends launcher           *
-     * Invisible UI                            */
-    { R"(\\Secret World Legends\\ClientPatcher\.exe$)", {{
-      { "d3d9.shaderModel",                 "2" },
+      { "d3d9.textureMemory",               "0"   },
     }} },
     /* Alien Rage                              *
      * GTX 295 & disable Hack to fix shadows   */
     { R"(\\(ShippingPC-AFEARGame|ARageMP)\.exe$)", {{
       { "d3d9.customVendorId",              "10de" },
       { "d3d9.customDeviceId",              "05E0" },
-      { "dxgi.nvapiHack",                   "False" },
-    }} },
-    /* BlazBlue Centralfiction                 *
-     * Temporary crash workaround              */
-    { R"(\\BBCF\.exe$)", {{
-      { "d3d9.textureMemory",               "0"   },
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
     /* Battle Fantasia Revised Edition         *
      * Speedup above 60fps                     */
     { R"(\\bf10\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
+    /* Codename Panzers Phase One/Two          *
+     * Main menu won't render after intros     */
+    { R"(\\(PANZERS|PANZERS_Phase_2)\.exe$)", {{
+      { "d3d9.enableDialogMode",            "True"   },
+    }} },
+    /* DC Universe Online                      *
+     * Freezes after alt tabbing               */
+    { R"(\\DCGAME\.EXE$)", {{
+      { "d3d9.deviceLostOnFocusLoss",       "True" },
+    }} },
+    /* Halo Online                             *
+     * Black textures                          */
+    { R"(\\eldorado\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict"   },
+    }} },
+    /* Injustice: Gods Among Us                *
+     * Locks a buffer that's still in use      */
+    { R"(\\injustice\.exe$)", {{
+      { "d3d9.allowDirectBufferMapping",    "False" },
+    }} },
+    /* STEINS;GATE ELITE                       */
+    { R"(\\SG_ELITE\\Game\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* The Incredibles                         */
+    { R"(\\IncPC\.exe$)", {{
+      { "d3d9.maxFrameRate",                "59" },
+    }} },
+    /* Conflict Vietnam                        */
+    { R"(\\Vietnam\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* Project: Snowblind                      */
+    { R"(\\Snowblind\.(SP|MP|exe)$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* Drakensang: The Dark Eye                */
+    { R"(\\drakensang\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    }} },
+
+    /**********************************************/
+    /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
+    /**********************************************/
+
+    /* Diablo 4 - Will complain about missing  *
+     * GPU unless dxgi Id match actual GPU Id  */
+    { R"(\\Diablo IV\.exe$)", {{
+      { "dxgi.hideNvidiaGpu",               "False"  },
+    }} },
     /* WILD HEARTS™️                            *
      * D3D12 title using D3D11 device for      *
      * media texture creation, whereby a large *
@@ -735,30 +829,10 @@ namespace dxvk {
     { R"(\\WILD HEARTS(_Trial)?\.exe$)", {{
       { "dxvk.maxChunkSize",                 "4" },
     }} },
-    /* Codename Panzers Phase One/Two          *
-     * Main menu won't render after intros     */
-    { R"(\\(PANZERS|PANZERS_Phase_2)\.exe$)", {{
-      { "d3d9.enableDialogMode",         "True"   },
-    }} },
-    /* Dark Romance: Vampire in Love          *
-     * Works around black screen or blinking  */
-    { R"(\\(DarkRomance_VampireInLove_CE)\.exe$)", {{
-      { "d3d9.allowDirectBufferMapping", "False"   },
-    }} },
-    /* DC Universe Online                      *
-     * Freezes after alt tabbing               */
-    { R"(\\DCGAME\.EXE$)", {{
-      { "d3d9.deviceLost",              "True"   },
-    }} },
-    
-    /**********************************************/
-    /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
-    /**********************************************/
-    
-    /* Diablo 4 - Will complain about missing  *
-     * GPU unless dxgi Id match actual GPU Id  */
-    { R"(\\Diablo IV\.exe$)", {{
-      { "dxgi.nvapiHack",                "False"  },
+    /* Ratchet & Clank: Rift Apart - does not allow
+     * enabling ray tracing if it sees an AMD GPU. */
+    { R"(\\RiftApart\.exe$)", {{
+      { "dxgi.hideNvidiaGpu",               "False" },
     }} },
   }};
 
@@ -767,7 +841,7 @@ namespace dxvk {
     return ch == ' ' || ch == '\x9' || ch == '\r';
   }
 
-  
+
   static bool isValidKeyChar(char ch) {
     return (ch >= '0' && ch <= '9')
         || (ch >= 'A' && ch <= 'Z')
@@ -804,12 +878,12 @@ namespace dxvk {
 
       while (n < e)
         key << line[n++];
-      
+
       ctx.active = key.str() == env::getExeName();
     } else {
       while (n < line.size() && isValidKeyChar(line[n]))
         key << line[n++];
-      
+
       // Check whether the next char is a '='
       n = skipWhitespace(line, n);
       if (n >= line.size() || line[n] != '=')
@@ -829,7 +903,7 @@ namespace dxvk {
         } else
           value << line[n++];
       }
-      
+
       if (ctx.active)
         config.setOption(key.str(), value.str());
     }
@@ -889,7 +963,7 @@ namespace dxvk {
           int32_t&      result) {
     if (value.size() == 0)
       return false;
-    
+
     // Parse sign, don't allow '+'
     int32_t sign = 1;
     size_t start = 0;
@@ -905,7 +979,7 @@ namespace dxvk {
     for (size_t i = start; i < value.size(); i++) {
       if (value[i] < '0' || value[i] > '9')
         return false;
-      
+
       intval *= 10;
       intval += value[i] - '0';
     }
@@ -914,8 +988,8 @@ namespace dxvk {
     result = sign * intval;
     return true;
   }
-  
-  
+
+
   bool Config::parseOptionValue(
     const std::string&  value,
           float&        result) {
@@ -1017,7 +1091,7 @@ namespace dxvk {
         std::regex expr(pair.first, std::regex::extended | std::regex::icase);
         return std::regex_search(appName, expr);
       });
-    
+
     if (appConfig != g_appDefaults.end()) {
       // Inform the user that we loaded a default config
       Logger::info(str::format("Found built-in config:"));
@@ -1037,30 +1111,42 @@ namespace dxvk {
 
     // Load either $DXVK_CONFIG_FILE or $PWD/dxvk.conf
     std::string filePath = env::getEnvVar("DXVK_CONFIG_FILE");
+    std::string confLine = env::getEnvVar("DXVK_CONFIG");
 
     if (filePath == "")
       filePath = "dxvk.conf";
-    
+
     // Open the file if it exists
     std::ifstream stream(str::topath(filePath.c_str()).c_str());
 
-    if (!stream)
+    if (!stream && confLine.empty())
       return config;
-    
-    // Inform the user that we loaded a file, might
-    // help when debugging configuration issues
-    Logger::info(str::format("Found config file: ", filePath));
 
     // Initialize parser context
     ConfigContext ctx;
     ctx.active = true;
 
-    // Parse the file line by line
-    std::string line;
+    if (stream) {
+      // Inform the user that we loaded a file, might
+      // help when debugging configuration issues
+      Logger::info(str::format("Found config file: ", filePath));
+
+      // Parse the file line by line
+      std::string line;
+
+      while (std::getline(stream, line))
+        parseUserConfigLine(config, ctx, line);
+    }
+
+    if (!confLine.empty()) {
+      // Inform the user that we parsing config from environment, might
+      // help when debugging configuration issues
+      Logger::info(str::format("Found config env: ", confLine));
+
+      for(auto l : str::split(confLine, ";"))
+        parseUserConfigLine(config, ctx, std::string(l.data(), l.size()));
+    }
 
-    while (std::getline(stream, line))
-      parseUserConfigLine(config, ctx, line);
-    
     return config;
   }
 
